package com.malk.kaiyue.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.NumberUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.malk.kaiyue.entity.AdvancedLeave;
import com.malk.kaiyue.mapper.AdvancedLeaveMapper;
import com.malk.kaiyue.service.KYCDService;
import com.malk.server.common.McR;
import com.malk.server.dingtalk.DDConf;
import com.malk.server.dingtalk.DDR;
import com.malk.server.dingtalk.DDR_New;
import com.malk.service.dingtalk.DDClient;
import com.malk.utils.UtilHttp;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;
@Slf4j
@Service
public class KYCDServiceImpl extends ServiceImpl<AdvancedLeaveMapper, AdvancedLeave> implements KYCDService {
    @Autowired
    private DDClient ddClient;

    @Autowired
    private DDConf ddConf;

    @Autowired
    private AdvancedLeaveMapper advancedLeaveMapper;

    @Value("${dingtalk_cd.appKey}")
    private String appKey;

    @Value("${dingtalk_cd.appSecret}")
    private String appSecret;

    @Value("${dingtalk_cd.agentId}")
    private String agentId;

    @Value("${dingtalk_cd.operator}")
    private String opUserId;

    //成都凯悦-年假(测试)
    private static final String LEAVE_CODE = "609a84ed-54d4-4ecd-a44f-4c55b04c37ea";
    //体验社-成都年假测试
//    private static final String LEAVE_CODE = "609a84ed-54d4-4ecd-a44f-4c55b04c37ea";

    @Override
    public List<String> getEmployeeUserId() {
        Map<String,Object> map = new HashMap<>();
        //在职员工状态筛选，可以查询多个状态。不同状态之间使用英文逗号分隔。
        //2：试用期  3：正式  5：待离职  -1：无状态
        map.put("status_list","3");
        //分页游标，从0开始。根据返回结果里的next_cursor是否为空来判断是否还有下一页，且再次调用时offset设置成next_cursor的值。
        map.put("offset",0);
        //分页大小，最大50。
        map.put("size",50);

        //获取员工userId集合
        List<String> userIdList = new ArrayList<>();
        getUserIdList(map,userIdList);

        return userIdList;
    }

    public List<String> getUserIdList(Map map,List<String> userIdList){
        //获取accessToken
        String access_token = ddClient.getAccessToken(appKey,appSecret);
        //调用钉钉接口获取在职员工userId集合
        DDR ddr = (DDR) UtilHttp.doPost("https://oapi.dingtalk.com/topapi/smartwork/hrm/employee/queryonjob", null, DDConf.initTokenParams(access_token), map, DDR.class);
        Map result = (Map) ddr.getResult();
        //将返回结果里的data_list合并到userIdList
        userIdList.addAll((List<String>) result.get("data_list"));
        //判断是否还有下一页
        if (Objects.nonNull(result.get("next_cursor"))){
            map.put("offset",result.get("next_cursor"));
            //递归将集合合并到userIdList
            getUserIdList(map,userIdList);
        }

        return userIdList;
    }

    @Override
    public List<Map> getEmployeeRosterInfo(Map<String, Object> map) {
        //获取accessToken
        String access_token = ddClient.getAccessToken(appKey,appSecret);
        //获取员工花名册字段信息
        if (Objects.nonNull(map)){
            DDR ddr = (DDR) UtilHttp.doPost("https://oapi.dingtalk.com/topapi/smartwork/hrm/employee/v2/list", null, DDConf.initTokenParams(access_token), map, DDR.class);
            return (List<Map>)ddr.getResult();
        }

        return null;
    }

    //保存10s内已处理的更新假期余额事件
    private Map<String, Long> bodyList = new HashMap<>();

    @Async
    public McR getEmployeeAnnualLeaveNum(Map<String, Object> map) {
        //获取accessToken
        String access_token = ddClient.getAccessToken(appKey,appSecret);

        //查询接口body添加参数
        //field_filter_list：要查询字段（花名册字段信息参考：https://open.dingtalk.com/document/orgapp/roster-custom-field-business-code）
        //agentid：企业内部应用AgentId
        map.put("field_filter_list","sys00-name,sys01-positionLevel,sys00-confirmJoinTime,b433c687-c3b3-4f97-8498-d23944f3316b,80292628-1c88-4c25-9c40-3e91283552e7");
        map.put("agentid",agentId);

        List<String> result = new ArrayList<>();
        //获取员工花名册字段信息
        if (Objects.nonNull(map)){
            DDR ddr = (DDR) UtilHttp.doPost("https://oapi.dingtalk.com/topapi/smartwork/hrm/employee/v2/list", null, DDConf.initTokenParams(access_token), map, DDR.class);
            List<Map> employeeData = (List<Map>) ddr.getResult();

            //遍历员工信息
            for (Map data : employeeData) {
                String userId = data.get("userid").toString();
                //入职日期
                String confirmJoinTime = "";
                //职级
                String positionLevel = "";
                //姓名
                String name = "";
                //原职级
                String oldPositionLevel = "";
                //升职日期
                String promotionTime = "";

                List<Map> fieldDataList = (List<Map>) data.get("field_data_list");
                for (Map fieldData : fieldDataList) {
                    String fieldCode = fieldData.get("field_code").toString();
                    List<Map> fieldValueList = (List<Map>) fieldData.get("field_value_list");

                    if (Objects.nonNull(fieldValueList.get(0).get("value"))){
                        String value = fieldValueList.get(0).get("value").toString();
                        switch (fieldCode){
                            case "sys00-confirmJoinTime": confirmJoinTime = value;break;
                            case "sys01-positionLevel": positionLevel = value;break;
                            case "sys00-name": name = value;break;
                            case "b433c687-c3b3-4f97-8498-d23944f3316b": oldPositionLevel = value;break;//成都原职级
//                            case "7482b192-9f1d-49fa-adab-6f33f7d0951e": oldPositionLevel = value;break;//高级假期原职级
                            case "80292628-1c88-4c25-9c40-3e91283552e7": promotionTime = value;break;//升职日期
                            default:break;
                        }
                    }
                }
                //若没有原职级 则默认原职级是现职级
                if ("".equals(oldPositionLevel)){
                    oldPositionLevel = positionLevel;
                }
                //若没有升职日期 则默认升职日期是入职日期
                if ("".equals(promotionTime)){
                    promotionTime = confirmJoinTime;
                }

                if ("".equals(confirmJoinTime) || "".equals(positionLevel) || "".equals(name) || "".equals(oldPositionLevel)){
                    log.info("更新员工年假余额：参数缺啦！");
                    return McR.errorParam("参数缺啦！");
                }


                //假期有效开始日期为当年1月1日
                DateTime beginDate = DateUtil.beginOfYear(new Date());
                //假期有效截至日期为当年12月31日
                DateTime endDate = DateUtil.endOfYear(new Date());
                //今天
                DateTime today = DateUtil.date();
                //当年天数
                int yearDays = DateUtil.dayOfYear(endDate);
                //年假
                double yearLeave = 0.0;
                //预支年假
                double advanceLeave = 0.0;


                //获取原职级年假基数
                int oldPositionLevelBaseNum = getAnnualLeaveBaseNum(oldPositionLevel);
                //获取现职级年假基数
                int positionLevelBaseNum = getAnnualLeaveBaseNum(positionLevel);

                //获取入职日期的月日
                String confirmJoinMonthAndDay = confirmJoinTime.substring(5);
                String year = DateUtil.year(new Date()) + "";
                //今年入职周年日
                DateTime anniversaryOfEmployment = DateUtil.parse(year + "-" + confirmJoinMonthAndDay);

                //计算今天是入职第几周年
                long anniversary = DateUtil.betweenYear(DateUtil.parse(confirmJoinTime), anniversaryOfEmployment, true) - 1;
                //判断员工是否当年新入职
                if (DateUtil.year(DateUtil.parse(confirmJoinTime)) == DateUtil.year(new Date())){
                    //分两段计算：入职日到升职日，升职日到年底
                    long day1 = DateUtil.betweenDay(DateUtil.parse(confirmJoinTime), DateUtil.parse(promotionTime), true);
                    long day2 = DateUtil.betweenDay(DateUtil.parse(promotionTime), endDate, true);
                    yearLeave = (double) (day1 * oldPositionLevelBaseNum + day2 * positionLevelBaseNum ) / yearDays;
                }else {
                    //升职日期在今年入职周年日之前
                    if (DateUtil.compare(anniversaryOfEmployment, DateUtil.parse(promotionTime)) >= 0){
                        //升职日期在今年
                        if (DateUtil.compare(DateUtil.parse(promotionTime),today) >= 0){
                            //获取年初到升职日的天数
                            int day1 = DateUtil.dayOfYear(DateUtil.parse(promotionTime));
                            //获升职值日到入职周年日的天数
                            long day2 = DateUtil.betweenDay(DateUtil.parse(promotionTime), anniversaryOfEmployment, true);
                            //获取入职周年日到年底的天数
                            long day3 = DateUtil.betweenDay(anniversaryOfEmployment, endDate, true);
                            //分三段计算：年初-升职日，升职日到入职周年日，入职周年日到年底
                            yearLeave = (double) (day1 * (oldPositionLevelBaseNum + (anniversary > 4 ? 4 : anniversary)) + day2 * (positionLevelBaseNum + (anniversary > 4 ? 4 : anniversary)) + day3 * (positionLevelBaseNum + (anniversary + 1 > 4 ? 4 : anniversary + 1))) / yearDays ;
                        }else{
                            //获取年初到入职周年日的天数
                            int day1 = DateUtil.dayOfYear(anniversaryOfEmployment);
                            //获取入职周年日到年底的天数
                            long day2 = DateUtil.betweenDay(anniversaryOfEmployment, endDate, true);
                            //分两段计算：年初到入职周年日，入职周年日到年底
                            yearLeave = (double) (day1 * (positionLevelBaseNum + (anniversary > 4 ? 4 : anniversary)) + day2 * (positionLevelBaseNum + (anniversary + 1 > 4 ? 4 : anniversary + 1))) / yearDays ;
                        }
                    }else {
                        //升职日在今年入职周年日之后
                        //获取年初到入职周年日的天数
                        int day1 = DateUtil.dayOfYear(anniversaryOfEmployment);
                        //获取入职周年日到升职日的天数
                        long day2 = DateUtil.betweenDay(anniversaryOfEmployment, DateUtil.parse(promotionTime), true);
                        //获取升职日到年底的天数
                        long day3 = DateUtil.betweenDay(DateUtil.parse(promotionTime), endDate, true);
                        //分三段计算：年初-入职周年日，入职周年日到升职日，升职日到年底
                        yearLeave = (double) (day1 * (oldPositionLevelBaseNum + (anniversary > 4 ? 4 : anniversary)) + day2 * (oldPositionLevelBaseNum + (anniversary + 1 > 4 ? 4 : anniversary + 1)) + day3 * (positionLevelBaseNum + (anniversary + 1 > 4 ? 4 : anniversary + 1))) / yearDays;
                    }
                }


                //年假小数
                /*double yearLeaveDecimalPart = yearLeave - (int) yearLeave;
                if (yearLeaveDecimalPart < 0.25){
                    yearLeave = (int) yearLeave;
                }else if (yearLeaveDecimalPart < 0.75){
                    yearLeave = (int) yearLeave + 0.5;
                }else if (yearLeaveDecimalPart < 1){
                    yearLeave = (int) yearLeave + 1;
                }*/
                yearLeave = Math.round(yearLeave * 100.0) / 100.0;

                //查询出用户消费年假记录 当返回leaveRecords中calType为null或不返回该字段则为请假消耗 将计算出的年假数减去请假消耗的数量
                Map body = new HashMap();
                body.put("opUserId",opUserId);
                body.put("leaveCode",LEAVE_CODE);
                body.put("userIds",new String[]{userId});
                body.put("pageNumber",0);
                body.put("pageSize",50);
                DDR_New useDdr = (DDR_New) UtilHttp.doPost("https://api.dingtalk.com/v1.0/attendance/vacations/records/query", DDConf.initTokenHeader(access_token), null, body, DDR_New.class);
                Map useResult = (Map) useDdr.getResult();
                List<Map> useList = (List<Map>) useResult.get("leaveRecords");
                Double useLeaveNum = 0d;
                if (Objects.nonNull(useList) && !useList.isEmpty()){
                    for (Map use : useList) {
                        DateTime gmtCreate = DateUtil.date((long) use.get("gmtCreate"));
                        //获取今年请假记录
                        if (DateUtil.year(gmtCreate) == DateUtil.year(new Date())){
                            //判断是否为正常请假而不是接口测试或期初假期发放
                            if (!"接口测试修改".equals(use.get("leaveReason").toString()) && !"期初假期发放".equals(use.get("leaveReason").toString())){
                                //若是请假消耗或管理员手动减少
                                if (!use.containsKey("calType") || Objects.isNull(use.get("calType")) || "delete".equals(use.get("calType").toString())){
                                    useLeaveNum += (int) use.get("recordNumPerDay") / 100;
                                }
                                //注：若是管理员手动增加 则假期余额会多出一个BCXsunNm记录增加的假期天数  最终会在设置的假期余额的基础上加上这些天数
                                //故此处手动新增的假期余额不做处理
                            }
                        }
                    }
                }

                LambdaQueryWrapper<AdvancedLeave> advancedLeaveLambdaQueryWrapper = new LambdaQueryWrapper<>();;
                advancedLeaveLambdaQueryWrapper.eq(AdvancedLeave::getUserId,userId)
                        .eq(AdvancedLeave::getValidFlag,"1");
                AdvancedLeave advancedLeave = advancedLeaveMapper.selectOne(advancedLeaveLambdaQueryWrapper);
                if (Objects.nonNull(advancedLeave)){
                    advanceLeave = Double.valueOf(advancedLeave.getLeaveNum());
                }else {
                    advanceLeave = NumberUtil.sub(yearLeave - useLeaveNum).doubleValue() >= 8 ? 8 : NumberUtil.sub(yearLeave - useLeaveNum).intValue();
                    advancedLeave.setLeaveNum(advanceLeave + "");
                    advancedLeave.setUserId(userId);
                    advancedLeave.setYear(String.valueOf(DateUtil.year(new Date())));
                    advancedLeaveMapper.insert(advancedLeave);
                }

                //年假总数
                int realYearLeave =  NumberUtil.sub(yearLeave, useLeaveNum, -advanceLeave).doubleValue() < 0 ? 0 : NumberUtil.sub(yearLeave, useLeaveNum, -advanceLeave).multiply(new BigDecimal(100)).intValue();

                //获取员工原年假余额 取较大值
                Map balanceMap = new HashMap();
                balanceMap.put("leave_code",LEAVE_CODE);
                balanceMap.put("op_userid",opUserId);
                balanceMap.put("userids",userId);
                balanceMap.put("offset",0);
                balanceMap.put("size",10);
                DDR balanceDdr = (DDR) UtilHttp.doPost("https://oapi.dingtalk.com/topapi/attendance/vacation/quota/list", null, DDConf.initTokenParams(access_token), balanceMap, DDR.class);
                Map balanceResult = (Map) balanceDdr.getResult();
                List<Map> leaveQuotas = (List<Map>) balanceResult.get("leave_quotas");
                if (Objects.nonNull(leaveQuotas) && !leaveQuotas.isEmpty()){
                    for (Map leaveQuota : leaveQuotas) {
                        if (year.equals(leaveQuota.get("quota_cycle"))){
                            int balance = (int) leaveQuota.get("quota_num_per_day");
                            if (balance > realYearLeave){
                                realYearLeave = balance;
                            }
                        }
                    }
                }


                //更新假期余额接口的body
                DateTime deadline = DateUtil.parse((DateUtil.year(new Date()) + 1) + "-03-31");
                Map<String,Object> updateBody = new HashMap<>();
                Map<String,Object> leave_quotas = new HashMap<>();
                //额度有效期开始时间，毫秒级时间戳
                leave_quotas.put("start_time",beginDate.getTime());
                //额度有效期结束时间，毫秒级时间戳。
                leave_quotas.put("end_time",deadline.getTime());
                //操作原因
                leave_quotas.put("reason","接口测试修改");
                //以天计算的额度总数 假期类型按天计算时，该值不为空且按百分之一天折算。 例如：1000=10天。
                leave_quotas.put("quota_num_per_day",realYearLeave);
                //以小时计算的额度总数 假期类型按小时，计算该值不为空且按百分之一小时折算。例如：1000=10小时。
                leave_quotas.put("quota_num_per_hour",0);
                //额度所对应的周期，格式必须是"yyyy"，例如"2021"
                leave_quotas.put("quota_cycle",DateUtil.year(new Date())+"");
                //自定义添加的假期类型：年假开发测试的leave_code
                leave_quotas.put("leave_code",LEAVE_CODE);
                //要更新的员工的userId
                leave_quotas.put("userid",userId);

                updateBody.put("leave_quotas",leave_quotas);
                //当前企业内拥有OA审批应用权限的管理员的userId
                updateBody.put("op_userid",opUserId);

                String bodyStr = realYearLeave + userId;

                // 检查更新事件是否已经处理过，如果是，则忽略该更新
                if (isUpdateLeave(bodyStr)) {
                    log.info("更新事件已处理，忽略该回调...");
                    return null;
                }

                // 将更新和当前时间戳添加到已处理集合中
                long currentTime = System.currentTimeMillis();
                bodyList.put(bodyStr, currentTime);

                result.add("姓名："+name+",职级：" + positionLevel+",原职级：" + oldPositionLevel+ ",年假基数：" + yearLeave + "天,入职日期：" + confirmJoinTime + ",升职日期：" + promotionTime + ",开始日期：" + beginDate+",截止日期：" + deadline);

                //更新假期余额
                UtilHttp.doPost("https://oapi.dingtalk.com/topapi/attendance/vacation/quota/update", null, DDConf.initTokenParams(access_token), updateBody, DDR.class);
            }
        }
        log.info(result.stream().collect(Collectors.joining(",")));
        return McR.success(result);
    }

    @Override
    public McR updateEmployeeAnnualLeaveNum() {
        //获取员工userId集合
        List<String> userIdList = getEmployeeUserId();
        //遍历集合给所有员工更新年假余额
        if (Objects.nonNull(userIdList) && !userIdList.isEmpty()){
            for (String userId : userIdList) {
                Map map = new HashMap();
                map.put("userid_list",userId);
                getEmployeeAnnualLeaveNum(map);
                //线程等待10s
                try {
                    log.info("线程睡眠10s");
                    Thread.sleep(10000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }
        return McR.success();
    }

    @Override
    public Map getUserLeaveInfo(String userId) {

        /*LambdaQueryWrapper<AdvancedLeave> advancedLeaveLambdaQueryWrapper = new LambdaQueryWrapper<>();
        advancedLeaveLambdaQueryWrapper.eq(AdvancedLeave::getUserId,userId)
                .eq(AdvancedLeave::getYear,String.valueOf(DateUtil.year(new Date())))
                .eq(AdvancedLeave::getValidFlag,"1");
        AdvancedLeave advancedLeave = advancedLeaveMapper.selectOne(advancedLeaveLambdaQueryWrapper);
*/
        long currentTime = System.currentTimeMillis();

        Map leaveMap = new HashMap();

        //年假余额
        BigDecimal annualLeaveNum = new BigDecimal(0.00);

        //查询员工年假余额
        List<Map> leaveQuotasList = new ArrayList<>();
        getLeaveNum("f7e0de92-806f-457e-9aa1-c20b245d741e",userId,0,50,leaveQuotasList);

        if (Objects.nonNull(leaveQuotasList)){
            for (Map leaveQuotas : leaveQuotasList) {
                if ((long) leaveQuotas.get("start_time") <= currentTime && currentTime <= (long) leaveQuotas.get("end_time")){
                    if (Objects.isNull(leaveQuotas.get("quota_num_per_hour")) && Objects.nonNull(leaveQuotas.get("quota_num_per_day"))){
                        annualLeaveNum = annualLeaveNum.add(new BigDecimal(String.valueOf(leaveQuotas.get("quota_num_per_day"))).divide(new BigDecimal(100)));
                   }
                    if (Objects.nonNull(leaveQuotas.get("used_num_per_day"))){
                        annualLeaveNum = annualLeaveNum.subtract(new BigDecimal(String.valueOf(leaveQuotas.get("used_num_per_day"))).divide(new BigDecimal(100)));
                   }
                }
            }
        }

        if (annualLeaveNum.compareTo(new BigDecimal(8)) >= 0){
            annualLeaveNum = annualLeaveNum.subtract(new BigDecimal(8));
            leaveMap.put("可预支年假",8);
            leaveMap.put("实际年假余额",annualLeaveNum);
            leaveMap.put("已预支年假",0);
        }else {
            BigDecimal relYearLeave = annualLeaveNum.remainder(new BigDecimal(0.5));
            leaveMap.put("可预支年假",annualLeaveNum.subtract(relYearLeave));
            leaveMap.put("实际年假余额",annualLeaveNum.subtract(new BigDecimal(8)));
            leaveMap.put("已预支年假",new BigDecimal(8).subtract(annualLeaveNum).add(relYearLeave));
        }

        //获取员工调休假余额
        List<Map> leaveQuotasList2 = new ArrayList<>();
        getLeaveNum("c7beb5e9-73ee-45f8-ae66-dfdb2a5ad0b8",userId,0,10,leaveQuotasList2);

        //调休假余额
        BigDecimal compensatoryLeaveNum = new BigDecimal(0.00);
        if (Objects.nonNull(leaveQuotasList2)){
            for (Map leaveQuotas : leaveQuotasList2) {
                if ((long) leaveQuotas.get("start_time") <= currentTime && currentTime <= (long) leaveQuotas.get("end_time")){
                    if (Objects.isNull(leaveQuotas.get("quota_num_per_day")) && Objects.nonNull(leaveQuotas.get("quota_num_per_hour"))){
                        compensatoryLeaveNum = compensatoryLeaveNum.add(new BigDecimal(String.valueOf(leaveQuotas.get("quota_num_per_hour"))).divide(new BigDecimal(100)));
                    }
                    if (Objects.nonNull(leaveQuotas.get("used_num_per_hour"))){
                        compensatoryLeaveNum = compensatoryLeaveNum.subtract(new BigDecimal(String.valueOf(leaveQuotas.get("used_num_per_hour"))).divide(new BigDecimal(100)));
                    }
                }
            }
        }
        if (compensatoryLeaveNum.compareTo(new BigDecimal(80)) >= 0){
            leaveMap.put("可预支调休",80);
            leaveMap.put("实际加班小时数",compensatoryLeaveNum.subtract(new BigDecimal(80)));
            leaveMap.put("已预支调休",0);
        }else {
            leaveMap.put("可预支调休",compensatoryLeaveNum);
            leaveMap.put("实际加班小时数",0);
            leaveMap.put("已预支调休",new BigDecimal(80).subtract(compensatoryLeaveNum));
        }

        return leaveMap;
    }

    private List<Map> getLeaveNum(String leave_code,String userId,int offset,int size,List<Map> leaveQuotasList) {
        String access_token = ddClient.getAccessToken(appKey,appSecret);

        Map body = new HashMap();
        body.put("leave_code",leave_code);
        body.put("op_userid",opUserId);
        body.put("userids",userId);
        body.put("offset",offset);
        body.put("size",size);

        DDR_New ddrNew = (DDR_New) UtilHttp.doPost("https://oapi.dingtalk.com/topapi/attendance/vacation/quota/list", null, DDConf.initTokenParams(access_token), body, DDR_New.class);
        Map result = (Map) ddrNew.getResult();

        if (Objects.nonNull(result.get("leave_quotas"))){
            leaveQuotasList.addAll((List<Map>) result.get("leave_quotas"));
        }

        if ((boolean) result.get("has_more")){
            getLeaveNum(leave_code,userId,offset+size,size,leaveQuotasList);
        }
        return leaveQuotasList;
    }

    private int getAnnualLeaveBaseNum(String positionLevel) {
        int annualLeave = 0;
        if (positionLevel.equals("MGR") || positionLevel.equals("DH") || positionLevel.equals("副主任医师")){
            annualLeave = 12;
        }else if (positionLevel.equals("LH") || positionLevel.equals("技术专员")){
            annualLeave = 10;
        }else if (positionLevel.equals("HST") || positionLevel.equals("高级管理员")){
            annualLeave = 8;
        }else if (positionLevel.equals("T") || positionLevel.equals("技术专家")){
            annualLeave = 0;
        }
        return annualLeave;
    }

    /**
     * 检查该更新事件在5s内是否处理过，应对钉钉瞬间重复回调
     *
     * @param msg 回调事件
     * @return 是否处理过
     */
    private boolean isUpdateLeave(String msg) {
        // 清理超过10s的回调事件
        long currentTime = System.currentTimeMillis();
        long expirationTime = currentTime - TimeUnit.SECONDS.toMillis(5);
        bodyList.entrySet().removeIf(entry -> entry.getValue() < expirationTime);

        return bodyList.containsKey(msg);
    }

}
