package com.malk.kaiyue.service.impl;

import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.malk.kaiyue.entity.AdvancedLeave;
import com.malk.kaiyue.mapper.AdvancedLeaveMapper;
import com.malk.kaiyue.service.KYYTService;
import com.malk.server.common.McR;
import com.malk.server.dingtalk.DDConf;
import com.malk.server.dingtalk.DDR;
import com.malk.server.dingtalk.DDR_New;
import com.malk.service.dingtalk.DDClient;
import com.malk.utils.UtilHttp;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;

@Slf4j
@Service
public class KYYTServiceImpl implements KYYTService {
    @Autowired
    private DDClient ddClient;

    @Autowired
    private DDConf ddConf;

    @Autowired
    private AdvancedLeaveMapper advancedLeaveMapper;

    @Value("${dingtalk_yt.appKey}")
    private String appKey;

    @Value("${dingtalk_yt.appSecret}")
    private String appSecret;

    @Value("${dingtalk_yt.agentId}")
    private String agentId;

    @Value("${dingtalk_yt.operator}")
    private String opUserId;


    //烟台凯悦-年假(测试)
//    private static final String LEAVE_CODE = "28abb7bf-e1b6-4387-9e2a-e1b2ae983e7a";
    //年假
    private static final String LEAVE_CODE = "3dcd4b8c-1b88-4206-9b52-cc9e2fde2e38";
    //体验社-年假测试wzy
//    private static final String LEAVE_CODE = "f9240c02-8fe7-4535-af2c-ca6740e1c654";

    @Override
    public List<Map> getEmployeeRosterInfo(Map<String, Object> map) {
        //获取accessToken
        String access_token = ddClient.getAccessToken(appKey,appSecret);
        //获取员工花名册字段信息
        if (Objects.nonNull(map)){
            DDR ddr = (DDR) UtilHttp.doPost("https://oapi.dingtalk.com/topapi/smartwork/hrm/employee/v2/list", null, DDConf.initTokenParams(access_token), map, DDR.class);
            return (List<Map>)ddr.getResult();
        }

        return null;
    }

    @Override
    public List<String> getEmployeeUserId() {
        Map<String,Object> map = new HashMap<>();
        //在职员工状态筛选，可以查询多个状态。不同状态之间使用英文逗号分隔。
        //2：试用期  3：正式  5：待离职  -1：无状态
        map.put("status_list","3");
        //分页游标，从0开始。根据返回结果里的next_cursor是否为空来判断是否还有下一页，且再次调用时offset设置成next_cursor的值。
        map.put("offset",0);
        //分页大小，最大50。
        map.put("size",50);

        //获取员工userId集合
        List<String> userIdList = new ArrayList<>();
        getUserIdList(map,userIdList);

        return userIdList;
    }

    public List<String> getUserIdList(Map map,List<String> userIdList){
        //获取accessToken
        String access_token = ddClient.getAccessToken(appKey,appSecret);
        //调用钉钉接口获取在职员工userId集合
        DDR ddr = (DDR) UtilHttp.doPost("https://oapi.dingtalk.com/topapi/smartwork/hrm/employee/queryonjob", null, DDConf.initTokenParams(access_token), map, DDR.class);
        Map result = (Map) ddr.getResult();
        //将返回结果里的data_list合并到userIdList
        userIdList.addAll((List<String>) result.get("data_list"));
        //判断是否还有下一页
        if (Objects.nonNull(result.get("next_cursor"))){
            map.put("offset",result.get("next_cursor"));
            //递归将集合合并到userIdList
            getUserIdList(map,userIdList);
        }

        return userIdList;
    }

    //保存5s内已处理的更新假期余额事件
    private Map<String, Long> bodyList = new HashMap<>();

    @Async
    public McR getEmployeeAnnualLeaveNum(Map<String, Object> map) {
        //获取accessToken
        String access_token = ddClient.getAccessToken(appKey,appSecret);

        //查询接口body添加参数
        //field_filter_list：要查询字段（花名册字段信息参考：https://open.dingtalk.com/document/orgapp/roster-custom-field-business-code）
        //agentid：企业内部应用AgentId
        map.put("field_filter_list","sys00-name,sys00-confirmJoinTime,sys01-positionLevel," +
                    "sys01-employeeType,sys02-joinWorkingTime,sys05-contractRenewCount," +
                    "sys05-nowContractStartTime,2a8b3a12-5102-4e60-b5d2-5f3bca0f0b7c"
                );
        map.put("agentid",agentId);

        List<String> result = new ArrayList<>();
        //获取员工花名册字段信息
        if (Objects.nonNull(map)){
            DDR ddr = (DDR) UtilHttp.doPost("https://oapi.dingtalk.com/topapi/smartwork/hrm/employee/v2/list", null, DDConf.initTokenParams(access_token), map, DDR.class);
            List<Map> employeeData = (List<Map>) ddr.getResult();

            //遍历员工信息
            for (Map data : employeeData) {
                String userId = data.get("userid").toString();
                //首次参加工作日期（计算工龄）
                String joinWorkingTime = "";
                //入职日期
                String confirmJoinTime = "";
                //职级
                String positionLevel = "";
                //原职级
                String oldPositionLevel = "";
                //升职日期
                String promotionTime = "";
                //姓名
                String name = "";
                //合同续签次数
                int contractRenewCount = 0;
                //现合同开始日期
                String contractStartTime = "";
                //员工类型
                String employeeType = "";

                List<Map> fieldDataList = (List<Map>) data.get("field_data_list");
                for (Map fieldData : fieldDataList) {
                    String fieldCode = fieldData.get("field_code").toString();
                    List<Map> fieldValueList = (List<Map>) fieldData.get("field_value_list");
                    if (Objects.nonNull(fieldValueList.get(0).get("value"))){
                        String value = fieldValueList.get(0).get("value").toString();
                        switch (fieldCode){
                            case "sys00-name": name = value;break;
                            case "sys00-confirmJoinTime": confirmJoinTime = value;break;
                            case "sys01-positionLevel": positionLevel = value;break;
                            case "sys01-employeeType": employeeType = value;break;
                            case "sys01-promotionTime": promotionTime = value;break;
                            case "sys02-joinWorkingTime": joinWorkingTime = value;break;
                            case "sys05-contractRenewCount": contractRenewCount = Integer.valueOf(value);break;
                            case "sys05-nowContractStartTime": contractStartTime = value;break;
                            case "2a8b3a12-5102-4e60-b5d2-5f3bca0f0b7c": oldPositionLevel = value;break;
                            default:break;
                        }
                    }

                }

                //若没有原职级 则默认原职级是现职级
                if ("".equals(oldPositionLevel)){
                    oldPositionLevel = positionLevel;
                }
                //若没有升职日期 则默认升职日期是入职日期
                if ("".equals(promotionTime)){
                    promotionTime = confirmJoinTime;
                }
                //若没有现合同开始日期 则默认现合同开始日期是入职日期
                if ("".equals(contractStartTime)){
                    contractStartTime = confirmJoinTime;
                }

                if ("".equals(confirmJoinTime) || "".equals(positionLevel) || "".equals(name)){
                    log.info("更新员工userid：{} 参数缺失！", userId);
                    continue;
                }

                //假期有效开始日期为当年1月1日
                DateTime beginDate = DateUtil.beginOfYear(new Date());
                //假期有效截至日期为当年12月31日
                DateTime endDate = DateUtil.beginOfDay(DateUtil.endOfYear(new Date()));
                //当年天数
                int yearDays = DateUtil.dayOfYear(endDate);

                //判断员工是否当年新入职
                if (DateUtil.year(DateUtil.parse(confirmJoinTime)) == DateUtil.year(new Date())){
                    beginDate = DateUtil.parse(confirmJoinTime);
                }

                //工龄（年） 计算规则：首次工作时间至当年一月一日 数值向下取整
                int workAge = 0;
                if (joinWorkingTime.equals("")){
                    workAge = 0;
                }else {
                    workAge =(int) (DateUtil.betweenYear(DateUtil.parse(joinWorkingTime), beginDate, true));
                    if (DateUtil.dayOfYear(DateUtil.parse(joinWorkingTime)) != 1 && workAge != 0){
                        workAge --;
                    }
                }

                System.out.println("截至今年1月1日，工龄为："+workAge + "年");
                //年假基数
                double yearLeave = 0.0;
                //预支年假
                double advanceLeave = 8.0;

                Date today = new Date();

                int year = DateUtil.year(new Date());

                //退休返聘年假基数5天
                if (employeeType.equals("5")){
                    double yearLeaveBaseNum = 5.0;
                    long day1 = DateUtil.betweenDay(beginDate, endDate, true);
                    yearLeave = (day1 * yearLeaveBaseNum) / yearDays;
                    log.info("day1:{},yearLeave:{}", day1, yearLeave);
                }else {
                    int positionLevelBaseNum = getAnnualLeaveBaseNum(positionLevel,workAge,contractRenewCount);
                    int oldPositionLevelBaseNum = getAnnualLeaveBaseNum(oldPositionLevel,workAge,contractRenewCount);

                    boolean flag1 = false;
                    boolean flag2 = false;
                    //判断升职日期是否是今年
                    if (DateUtil.year(DateUtil.parse(promotionTime)) == year){
                        flag1 = true;
                    }
                    //判断合同开始日期是否在今年
                    if (DateUtil.year(DateUtil.parse(contractStartTime)) == year){
                        flag2 = true;
                    }
                    //上期合同续签数附加年假数
                    int lastAdditiveYearLeave = Math.min(Math.max(contractRenewCount - 1, 0), 2) * 2;
                    //合同续签数附加年假数
                    int additiveYearLeave = Math.min(contractRenewCount, 2) * 2;
                    if (!flag1 && !flag2){
                        long day1 = DateUtil.betweenDay(beginDate, endDate, true);
                        yearLeave = (double) (day1 * (positionLevelBaseNum + additiveYearLeave)) / yearDays ;
                        log.info("day1:{},yearLeave:{}", day1, yearLeave);
                    }
                    if (flag1 && !flag2){
                        //获取假期有效开始日期到升职日的天数
                        long day1 = DateUtil.betweenDay(beginDate, DateUtil.parse(promotionTime), true);
                        //获取升职日到假期截止时间的天数
                        long day2 = DateUtil.betweenDay(DateUtil.parse(promotionTime), endDate, true) + 1;
                        //分两段计算：假期有效开始日期-现升职日，现升职日到假期截止时间
                        yearLeave = (double) (day1 * (oldPositionLevelBaseNum + additiveYearLeave) + day2 * (positionLevelBaseNum + additiveYearLeave)) / yearDays ;
                        log.info("day1:{},day2:{},yearLeave:{}", day1, day2, yearLeave);
                    }
                    if (!flag1 && flag2){
                        //获取假期有效开始日期到现合同开始日的天数
                        long day1 = DateUtil.betweenDay(beginDate, DateUtil.parse(contractStartTime), true);
                        //获取现合同开始日到今天的天数
                        long day2 = DateUtil.betweenDay(DateUtil.parse(contractStartTime), endDate, true) + 1;
                        //分两段计算：假期有效开始日期-现合同开始日期，现合同开始日期到假期截止时间
                        yearLeave = (double) (day1 * (positionLevelBaseNum + lastAdditiveYearLeave) + day2 * (positionLevelBaseNum + additiveYearLeave)) / yearDays;
                        log.info("day1:{},day2:{},yearLeave:{}", day1, day2, yearLeave);
                    }
                    if (flag1 && flag2){
                        //判断升职日期与现合同开始日期先后
                        //升职日期在现合同开始日期之前
                        if (DateUtil.compare(DateUtil.parse(contractStartTime), DateUtil.parse(promotionTime)) >= 0){
                            //获取假期有效开始日期到升职日的天数
                            long day1 = DateUtil.betweenDay(beginDate, DateUtil.parse(promotionTime), true);
                            //获升职值日到现合同开始日期的天数
                            long day2 = DateUtil.betweenDay(DateUtil.parse(promotionTime), DateUtil.parse(contractStartTime), true);
                            //获取现合同开始日期到假期截止时间的天数
                            long day3 = DateUtil.betweenDay(DateUtil.parse(contractStartTime), endDate, true) + 1;
                            //分三段计算：假期有效开始日期-现升职日，现职日到现现合同开始日期，现合同开始日期到假期截止时间
                            yearLeave = (double) (day1 * (oldPositionLevelBaseNum + lastAdditiveYearLeave) + day2 * (positionLevelBaseNum + lastAdditiveYearLeave) + day3 * (positionLevelBaseNum + additiveYearLeave)) / yearDays ;
                            log.info("day1:{},day2:{},day3:{},yearLeave:{}", day1, day2, day3, yearLeave);
                        }else {
                            //升职日在现合同开始日期之后
                            //获取假期有效开始日期到现合同开始日的天数
                            long day1 = DateUtil.betweenDay(beginDate, DateUtil.parse(contractStartTime), true);
                            //获取现合同开始日到升职日的天数
                            long day2 = DateUtil.betweenDay(DateUtil.parse(contractStartTime), DateUtil.parse(promotionTime), true);
                            //获取升职日到假期截止时间的天数
                            long day3 = DateUtil.betweenDay(DateUtil.parse(promotionTime), endDate, true) + 1;
                            //分三段计算：假期有效开始日期-现合同开始日期，现合同开始日期到升职日，升职日到假期截止时间
                            yearLeave = (double) (day1 * (oldPositionLevelBaseNum + lastAdditiveYearLeave) + day2 * (oldPositionLevelBaseNum + additiveYearLeave) + day3 * (positionLevelBaseNum + additiveYearLeave)) / yearDays;
                            log.info("day1:{},day2:{},day3:{},yearLeave:{}", day1, day2, day3, yearLeave);}
                    }

                    /*//判断员工是否当年新入职
                    if (DateUtil.year(DateUtil.parse(confirmJoinTime)) == DateUtil.year(today)){
                        day1 = DateUtil.betweenDay(DateUtil.parse(confirmJoinTime),new Date(),true);
                        day2 = DateUtil.betweenDay(today,endDate,true);
                    }else {
                        day1 = DateUtil.betweenDay(beginDate,today,true);
                        day2 = DateUtil.betweenDay(today,endDate,true);
                    }
                    yearLeave = (double) (day1 * oldYearLeaveBaseNum + day2 * yearLeaveBaseNum) / yearDays;
                    */
                }

                double yearLeaveDecimalPart = yearLeave - (int) yearLeave;

                result.add("姓名："+name+",职级："+positionLevel+",工龄："+workAge+"年,合同续签数"+contractRenewCount+",年假数：" + yearLeave + "天"+",截止日期："+endDate);

                if (yearLeaveDecimalPart < 0.25){
                    yearLeave = (int) yearLeave;
                }else if (yearLeaveDecimalPart < 0.75){
                    yearLeave = (int) yearLeave + 0.5;
                }else if (yearLeaveDecimalPart < 1){
                    yearLeave = (int) yearLeave + 1;
                }

                //查询出用户消费年假记录 当返回leaveRecords中calType为null或不返回该字段则为请假消耗 将计算出的年假数减去请假消耗的数量
                /*Map body = new HashMap();
                body.put("opUserId",opUserId);
                body.put("leaveCode",LEAVE_CODE);
                body.put("userIds",new String[]{userId});
                body.put("pageNumber",0);
                body.put("pageSize",50);
                DDR_New useDdr = (DDR_New) UtilHttp.doPost("https://api.dingtalk.com/v1.0/attendance/vacations/records/query", DDConf.initTokenHeader(access_token), null, body, DDR_New.class);
                Map useResult = (Map) useDdr.getResult();
                List<Map> useList = (List<Map>) useResult.get("leaveRecords");
                Double useLeaveNum = 0d;
                if (Objects.nonNull(useList) && !useList.isEmpty()){
                    for (Map use : useList) {
                        //判断是否为今年请假
                        DateTime gmtCreate = DateUtil.date((long) use.get("gmtCreate"));
                        if (DateUtil.year(gmtCreate) == DateUtil.year(new Date())){
                            //判断是否为正常请假而不是接口测试或期初假期发放
                            if (!"接口测试修改".equals(use.get("leaveReason").toString()) && !"期初假期发放".equals(use.get("leaveReason").toString())){
                                //若是请假消耗或管理员手动减少
                                if (!use.containsKey("calType") || Objects.isNull(use.get("calType")) || "delete".equals(use.get("calType").toString())){
                                    useLeaveNum += (int) use.get("recordNumPerDay") / 100;
                                }
                                //注：若是管理员手动增加 则假期余额会多出一个BCXsunNm记录增加的假期天数  最终会在设置的假期余额的基础上加上这些天数
                                //故此处手动新增的假期余额不做处理
                            }
                        }else if (DateUtil.year(gmtCreate) == (DateUtil.year(new Date()) - 1)){
                            //获取去年预支年假使用情况
                            if ("预支年假".equals(use.get("leaveReason").toString())){
                                //若去年预支过则在今年预支年假数中相应减少
                                if (!use.containsKey("calType") || Objects.isNull(use.get("calType")) || "delete".equals(use.get("calType").toString())){
                                    advanceLeave -= (int) use.get("recordNumPerDay") / 100;
                                }
                            }
                        }
                    }
                }*/

                //实际年假数（包含预支假）

                //年假余额
                BigDecimal annualLeaveNum = new BigDecimal(0.00);
                //查询员工年假余额
                long currentTime2 = System.currentTimeMillis();
                List<Map> leaveQuotasList = new ArrayList<>();
                getLeaveNum(LEAVE_CODE,userId,0,50,leaveQuotasList);
                if (Objects.nonNull(leaveQuotasList)){
                    for (Map leaveQuotas : leaveQuotasList) {
                        if ((long) leaveQuotas.get("start_time") <= currentTime2 && currentTime2 <= (long) leaveQuotas.get("end_time")){
                            if (Objects.isNull(leaveQuotas.get("quota_num_per_hour")) && Objects.nonNull(leaveQuotas.get("quota_num_per_day"))){
                                annualLeaveNum = annualLeaveNum.add(new BigDecimal(String.valueOf(leaveQuotas.get("quota_num_per_day"))).divide(new BigDecimal(100)));
                            }
                            if (Objects.nonNull(leaveQuotas.get("used_num_per_day"))){
                                annualLeaveNum = annualLeaveNum.subtract(new BigDecimal(String.valueOf(leaveQuotas.get("used_num_per_day"))).divide(new BigDecimal(100)));
                            }
                        }
                    }
                }
                LambdaQueryWrapper<AdvancedLeave> advancedLeaveLambdaQueryWrapper = new LambdaQueryWrapper<>();;
                advancedLeaveLambdaQueryWrapper.eq(AdvancedLeave::getUserId,userId)
                        .eq(AdvancedLeave::getYear,year)
                        .eq(AdvancedLeave::getValidFlag,"1")
                        .eq(AdvancedLeave::getCity,"yt");
                AdvancedLeave advancedLeave = advancedLeaveMapper.selectOne(advancedLeaveLambdaQueryWrapper);
                if (Objects.nonNull(advancedLeave)){
                    advanceLeave = Double.valueOf(advancedLeave.getLeaveNum());
                }
                else {
                    advancedLeave = new AdvancedLeave();
                    //年假余额大于8时 默认8
                    if (annualLeaveNum.compareTo(new BigDecimal(8)) > 0){
                        annualLeaveNum = new BigDecimal(8);
                    }
                    advanceLeave = Double.parseDouble(annualLeaveNum.subtract(annualLeaveNum.remainder(new BigDecimal(0.5))).toString());
                    advancedLeave.setLeaveNum(advanceLeave + "");
                    advancedLeave.setUserId(userId);
                    advancedLeave.setYear(String.valueOf(DateUtil.year(new Date())));
                    advancedLeave.setCity("yt");
                    advancedLeaveMapper.insert(advancedLeave);
                }
                double realYearLeave = (yearLeave + advanceLeave) < 0 ? 0 : (yearLeave + advanceLeave);


                //更新假期余额接口的body
                Map<String,Object> updateBody = new HashMap<>();
                Map<String,Object> leave_quotas = new HashMap<>();
                //额度有效期开始时间，毫秒级时间戳
                leave_quotas.put("start_time",beginDate.getTime());
                //额度有效期结束时间，毫秒级时间戳。
                leave_quotas.put("end_time",endDate.getTime());
                //操作原因
                leave_quotas.put("reason","接口年假发放");
                //以天计算的额度总数 假期类型按天计算时，该值不为空且按百分之一天折算。 例如：1000=10天。
                leave_quotas.put("quota_num_per_day",(int) (realYearLeave * 100) );
                //以小时计算的额度总数 假期类型按小时，计算该值不为空且按百分之一小时折算。例如：1000=10小时。
                leave_quotas.put("quota_num_per_hour",0);
                //额度所对应的周期，格式必须是"yyyy"，例如"2021"
                leave_quotas.put("quota_cycle",DateUtil.year(new Date())+"");
                //自定义添加的假期类型：年假开发测试的leave_code
                leave_quotas.put("leave_code",LEAVE_CODE);
                //要更新的员工的userId
                leave_quotas.put("userid",userId);

                updateBody.put("leave_quotas",leave_quotas);
                //当前企业内拥有OA审批应用权限的管理员的userId
                updateBody.put("op_userid",opUserId);

                String bodyStr = yearLeave + userId;

                // 检查更新事件是否已经处理过，如果是，则忽略该更新
                if (isUpdateLeave(bodyStr)) {
                    log.info("更新事件已处理，忽略该回调...");
                    return null;
                }


                // 将更新和当前时间戳添加到已处理集合中
                long currentTime = System.currentTimeMillis();
                bodyList.put(bodyStr, currentTime);

                //更新假期余额
                UtilHttp.doPost("https://oapi.dingtalk.com/topapi/attendance/vacation/quota/update", null, DDConf.initTokenParams(access_token), updateBody, DDR.class);
            }
        }
        log.info(result.stream().collect(Collectors.joining(",")));
        return McR.success(result);
    }

    private int getAnnualLeaveBaseNum(String positionLevel, int workAge, int contractRenewCount) {
        //法定年假
        int legalAnnualLeave = 0;
        //福利年假
        int welfareAnnualLeave = 0;
        //根据职级、工龄和合同续签数计算年假基数
        if (positionLevel.equals("5") || positionLevel.equals("副主任医师")){
            if (workAge < 10){
                legalAnnualLeave = 5;
                welfareAnnualLeave = 15;
            }else if (workAge >= 10 && workAge < 20){
                legalAnnualLeave = 10;
                welfareAnnualLeave = 10;
            }else if (workAge >= 20){
                legalAnnualLeave = 15;
                welfareAnnualLeave = 5;
            }
        }else if (positionLevel.equals("4") || positionLevel.equals("3") || positionLevel.equals("技术专员")){
            if (workAge < 10){
                legalAnnualLeave = 5;
                welfareAnnualLeave = 7 + 2 * (Math.min(contractRenewCount, 2));
            }else if (workAge >= 10 && workAge < 20){
                legalAnnualLeave = 10;
                welfareAnnualLeave = 2 + 2 * (Math.min(contractRenewCount, 2));
            }else if (workAge >= 20){
                legalAnnualLeave = 15;
                welfareAnnualLeave = 1;
            }
        }else if (positionLevel.equals("2") || positionLevel.equals("高级管理员")){
            if (workAge < 10){
                legalAnnualLeave = 5;
                welfareAnnualLeave = 5 + 2 * (Math.min(contractRenewCount, 2));
            }else if (workAge >= 10 && workAge < 20){
                legalAnnualLeave = 10;
                welfareAnnualLeave = 0 + 2 * (Math.min(contractRenewCount, 2));
            }else if (workAge >= 20){
                legalAnnualLeave = 15;
                welfareAnnualLeave = 0;
            }
        }else if (positionLevel.equals("1") || positionLevel.equals("技术专家")){
            if (workAge < 10){
                legalAnnualLeave = 5;
                welfareAnnualLeave = 3  + 2 * (Math.min(contractRenewCount, 2));
            }else if (workAge >= 10 && workAge < 20){
                legalAnnualLeave = 10;
                welfareAnnualLeave = 0  + 2 * (Math.min(contractRenewCount, 1));
            }else if (workAge >= 20){
                legalAnnualLeave = 15;
                welfareAnnualLeave = 0;
            }
        }else {
            legalAnnualLeave = 0;
            welfareAnnualLeave = 0;
        }
        int annualLeave = legalAnnualLeave + welfareAnnualLeave;
        return annualLeave;
    }

    @Override
    public McR updateEmployeeAnnualLeaveNum() {
        //获取员工userId集合
        List<String> userIdList = getEmployeeUserId();
        //遍历集合给所有员工更新年假余额
        if (Objects.nonNull(userIdList) && !userIdList.isEmpty()){
            for (String userId : userIdList) {
                Map map = new HashMap();
                map.put("userid_list",userId);
                getEmployeeAnnualLeaveNum(map);
                //线程等待10s
                try {
                    log.info("线程睡眠10s");
                    Thread.sleep(10000);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }
        return McR.success();
    }

    @Override
    public Map getUserLeaveInfo(String userId) {
        long currentTime = System.currentTimeMillis();

        Map leaveMap = new HashMap();

        //年假余额
        BigDecimal annualLeaveNum = new BigDecimal(0.00);

        //查询员工年假余额
        List<Map> leaveQuotasList = new ArrayList<>();
        getLeaveNum("3dcd4b8c-1b88-4206-9b52-cc9e2fde2e38",userId,0,50,leaveQuotasList);

        if (Objects.nonNull(leaveQuotasList)){
            for (Map leaveQuotas : leaveQuotasList) {
                if ((long) leaveQuotas.get("start_time") <= currentTime && currentTime <= (long) leaveQuotas.get("end_time")){
                    if (Objects.isNull(leaveQuotas.get("quota_num_per_hour")) && Objects.nonNull(leaveQuotas.get("quota_num_per_day"))){
                        annualLeaveNum = annualLeaveNum.add(new BigDecimal(String.valueOf(leaveQuotas.get("quota_num_per_day"))).divide(new BigDecimal(100)));
                    }
                    if (Objects.nonNull(leaveQuotas.get("used_num_per_day"))){
                        annualLeaveNum = annualLeaveNum.subtract(new BigDecimal(String.valueOf(leaveQuotas.get("used_num_per_day"))).divide(new BigDecimal(100)));
                    }
                }
            }
        }

        if (annualLeaveNum.compareTo(new BigDecimal(8)) == 1 || annualLeaveNum.compareTo(new BigDecimal(8)) == 0){
            annualLeaveNum = annualLeaveNum.subtract(new BigDecimal(8));
            leaveMap.put("可预支年假",8);
            leaveMap.put("实际年假余额",annualLeaveNum);
            leaveMap.put("已预支年假",0);
        }else {
            BigDecimal relYearLeave = annualLeaveNum.remainder(new BigDecimal(0.5));
            leaveMap.put("可预支年假",annualLeaveNum.subtract(relYearLeave));
            leaveMap.put("实际年假余额",annualLeaveNum.subtract(new BigDecimal(8)));
            leaveMap.put("已预支年假",new BigDecimal(8).subtract(annualLeaveNum).add(relYearLeave));
        }

        //获取员工调休假余额
        List<Map> leaveQuotasList2 = new ArrayList<>();
        getLeaveNum("b6301b0e-193a-4bce-b225-2f67a5d6244b",userId,0,10,leaveQuotasList2);

        //调休假余额
        BigDecimal compensatoryLeaveNum = new BigDecimal(0.00);
        if (Objects.nonNull(leaveQuotasList2)){
            for (Map leaveQuotas : leaveQuotasList2) {
                if ((long) leaveQuotas.get("start_time") <= currentTime && currentTime <= (long) leaveQuotas.get("end_time")){
                    if (Objects.isNull(leaveQuotas.get("quota_num_per_day")) && Objects.nonNull(leaveQuotas.get("quota_num_per_hour"))){
                        compensatoryLeaveNum = compensatoryLeaveNum.add(new BigDecimal(String.valueOf(leaveQuotas.get("quota_num_per_hour"))).divide(new BigDecimal(100)));
                    }
                    if (Objects.nonNull(leaveQuotas.get("used_num_per_hour"))){
                        compensatoryLeaveNum = compensatoryLeaveNum.subtract(new BigDecimal(String.valueOf(leaveQuotas.get("used_num_per_hour"))).divide(new BigDecimal(100)));
                    }
                }
            }
        }
        if (compensatoryLeaveNum.compareTo(new BigDecimal(80)) >= 0){
            leaveMap.put("可预支调休",80);
            leaveMap.put("实际加班小时数",compensatoryLeaveNum.subtract(new BigDecimal(80)));
            leaveMap.put("已预支调休",0);
        }else {
            leaveMap.put("可预支调休",compensatoryLeaveNum);
            leaveMap.put("实际加班小时数",0);
            leaveMap.put("已预支调休",new BigDecimal(80).subtract(compensatoryLeaveNum));
        }

        return leaveMap;

    }

    private List<Map> getLeaveNum(String leave_code,String userId,int offset,int size,List<Map> leaveQuotasList) {
        String access_token = ddClient.getAccessToken(appKey,appSecret);
        Map body = new HashMap();
        body.put("leave_code",leave_code);
        body.put("op_userid",opUserId);
        body.put("userids",userId);
        body.put("offset",offset);
        body.put("size",size);

        DDR_New ddrNew = (DDR_New) UtilHttp.doPost("https://oapi.dingtalk.com/topapi/attendance/vacation/quota/list", null, DDConf.initTokenParams(access_token), body, DDR_New.class);
        Map result = (Map) ddrNew.getResult();
        if (Objects.nonNull(result.get("leave_quotas"))){
            leaveQuotasList.addAll((List<Map>) result.get("leave_quotas"));
        }

        if ((boolean) result.get("has_more")){
            getLeaveNum(leave_code,userId,offset+size,size,leaveQuotasList);
        }
        return leaveQuotasList;
    }


    /**
     * 检查该更新事件在5s内是否处理过，应对钉钉瞬间重复回调
     *
     * @param msg 回调事件
     * @return 是否处理过
     */
    private boolean isUpdateLeave(String msg) {
        // 清理超过10s的回调事件
        long currentTime = System.currentTimeMillis();
        long expirationTime = currentTime - TimeUnit.SECONDS.toMillis(5);
        bodyList.entrySet().removeIf(entry -> entry.getValue() < expirationTime);

        return bodyList.containsKey(msg);
    }

}
